function [a,b] = trigpoly(x,y,k)           %last updated  12/29/99
%TRIG POLY  Compute interpolant or a least squares using trig. poly.
%           of degreee k. 
%
%           A graph of the original data and trig approximation is shown.
%           Pseudo inverse is used.
%
%    Use in the form ==>  [a,b] = trigpoly(x,y,k)  <==
%                 or              trigpoly(x,y,k)
%    INPUT:
%    x = vector of x-coordinates assumed to be in [-pi, pi]
%                                and assumed to in increasing order.
%
%    y = vector of y-coordinates
%
%    k = degree of the trig. polynomial (see below for conventions)
%
%    OUTPUT:
%                a = vector of coefficients of cosine terms
% 
%                b = vector of coefficients of sine terms
%
%    INFORMATION and CONVENTIONS:
%
%Let n be the number of data points. 
%If k = n/2, then we want to interpolate.
%
%We use the convention that cos(kt) is in the linear
%combination of terms, but sin(kt) is not. In addition to
%the constant term a(0) we will have a(k), but no b(k).
%
%So that the Fourier coefficient vectors have the same length
%we pad a zero to the front and back of the computed vector for b.
%
%The interpolant will be given as follows in terms of
%the vectors generated in this routine.
%
%            k+1
%     a(1) + SUM (a(j)*cos((j-1)t) + b(j)*sin((j-1)t)) 
%            j=2
%
%If k < n/2, then we want a least squares approximation.
%
%We use the convention that cos(kt) and sin(kt) are in the linear
%combination of terms. To account for the constant term a(0) 
%we pad b at the beginning with a zero value so the vectors have
%the same length. The preceding summation is valid for both
%interpolation and least squares approximations.(Note: Burden &
%Faires leaves off the sin(kt) term for least squares also.)
%
%  By: David R. Hill, MATH Department, Temple University
%      Philadelphia, Pa., 19122        Email: dhill001@temple.edu

%Converting data to a column vector
x=x(:);y=y(:);
n = length(x);

%Determing type of operation; interpolate or lsq
aptype='L'; %Least squares type
if k==n/2,aptype='I';end  %Interpolation type
%
%Starting to build the coefficient matrix of the normal system
%
A=ones(n,1);
%
%Loop for column of cos & sin
%
if aptype=='L'
   for j = 1:k
       A=[A cos(j*x) sin(j*x)];  %LSQ convention
   end
else
   for j = 1:k-1 % use k-1 here for ala Burden & Faires
       A=[A cos(j*x) sin(j*x)]; %INTERPOLATION convention
   end
   A=[A cos(k*x)]; %done ala Burden Faires
end
B=pinv(A); %computing the pseudo inverse 
c=B*y; %Getting the least squares on interpolation solution
%getting the coefficients
a=c(2:2:length(c));a=[c(1); a];
b=c(3:2:length(c));
if aptype=='I'
   b=[0;b;0];
else
   b=[0;b];
end
%
%Now we have the Fourier coefficients.
%
%Next we graph the original data set and the model curve
%
hstep=(x(n)-x(1))/100;
xorig=x(1):hstep:x(n);
v=a(1);
for j=2:k+1
    v=v+a(j)*cos((j-1)*xorig)+b(j)*sin((j-1)*xorig);
end
figure,plot(x,y,'m+',xorig,v,'b-')
title(['+''s are the data set; TRIG. POLY of degree ' num2str(k)])
xlabel('Press ENTER twice to continue.')
pause,clc,disp('TRIGPOLY is over!')

